# coding: utf-8

"""
    Curia Platform API

    These are the docs for the curia platform API. To test, generate an authorization token first.  # noqa: E501

    OpenAPI spec version: 3.8.1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import unittest

import curia.api.curia.api.curia.api.swagger_client
from curia.api.curia.api.curia.api.swagger_client.api.tasks_api import TasksApi  # noqa: E501
from curia.api.curia.api.curia.api.swagger_client.rest import ApiException


class TestTasksApi(unittest.TestCase):
    """TasksApi unit test stubs"""

    def setUp(self):
        self.api = TasksApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_create_many_base_task_controller_task(self):
        """Test case for create_many_base_task_controller_task

        Create multiple Tasks  # noqa: E501
        """
        pass

    def test_create_one_base_task_controller_task(self):
        """Test case for create_one_base_task_controller_task

        Create a single Task  # noqa: E501
        """
        pass

    def test_delete_one_base_task_controller_task(self):
        """Test case for delete_one_base_task_controller_task

        Delete a single Task  # noqa: E501
        """
        pass

    def test_get_many_base_task_controller_task(self):
        """Test case for get_many_base_task_controller_task

        Retrieve multiple Tasks  # noqa: E501
        """
        pass

    def test_get_one_base_task_controller_task(self):
        """Test case for get_one_base_task_controller_task

        Retrieve a single Task  # noqa: E501
        """
        pass

    def test_replace_one_base_task_controller_task(self):
        """Test case for replace_one_base_task_controller_task

        Replace a single Task  # noqa: E501
        """
        pass

    def test_task_controller_create_task_from_databricks_job(self):
        """Test case for task_controller_create_task_from_databricks_job

        Create task from Databricks job  # noqa: E501
        """
        pass

    def test_task_controller_get_awsecr_repos(self):
        """Test case for task_controller_get_awsecr_repos

        List ECR repos  # noqa: E501
        """
        pass

    def test_task_controller_list_databricks_jobs(self):
        """Test case for task_controller_list_databricks_jobs

        List Databricks jobs  # noqa: E501
        """
        pass

    def test_update_one_base_task_controller_task(self):
        """Test case for update_one_base_task_controller_task

        Update a single Task  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()
